@@ -29,7 +29,7 @@ use Image::ExifTool::IPTC;
 use Image::ExifTool::Canon;
 use Image::ExifTool::Nikon;
 
-$VERSION = '2.01';
+$VERSION = '2.02';
 @ISA = qw(Exporter);
 
 sub NumbersFirst;
@@ -217,6 +217,11 @@ format name.  For tags where a range of lengths is allowed, the minimum and
 maximum lengths are separated by a comma within the brackets.  IPTC strings
 are not null terminated.
 
+When writing, ExifTool issues a minor warning and truncates the value if it
+is longer than allowed by the IPTC specification.  Minor errors may be
+ignored to allow writing of longer values, but beware that values like this
+may cause problems for other IPTC readers.
+
 IPTC information is separated into different records, each of which has its
 own set of tags.
 
@@ -460,6 +465,7 @@ sub new
     }
 
     my $tableNum = 0;
+    my $exifTool = new Image::ExifTool;
     my ($tableName, $tag);
     # create lookup for short table names
     foreach $tableName (@tableNames) {
@@ -507,8 +513,9 @@ sub new
         $isIPTC = 1 if $writeProc and $writeProc eq \&Image::ExifTool::IPTC::WriteIPTC;
         $isXMP = 1 if $short =~ /^XMP\b/;
         $noID = 1 if $short =~ /^(XMP|Shortcuts|ASF.*)$/ or $$vars{NO_ID};
-        my $processBinaryData = ($$table{PROCESS_PROC} and
-            $$table{PROCESS_PROC} eq \&Image::ExifTool::ProcessBinaryData);
+        my $processBinaryData = ($$table{PROCESS_PROC} and (
+            $$table{PROCESS_PROC} eq \&Image::ExifTool::ProcessBinaryData or
+            $$table{PROCESS_PROC} eq \&Image::ExifTool::Nikon::ProcessNikonEncrypted));
         if ($$vars{ID_LABEL} or $processBinaryData) {
             $binaryTable = 1;
             $id{$tableName} = $$vars{ID_LABEL} || 'Index';
@@ -558,6 +565,25 @@ TagID:  foreach $tagID (@keys) {
                     warn "Warning: Hidden tag in list - $short $name\n" if @infoArray > 1;
                     next TagID;
                 }
+                my $writable;
+                if (defined $$tagInfo{Writable}) {
+                    $writable = $$tagInfo{Writable};
+                } elsif (not $$tagInfo{SubDirectory}) {
+                    $writable = $$table{WRITABLE};
+                }
+                # validate some characteristics of obvious date/time tags
+                if ($$tagInfo{PrintConv} and $$tagInfo{PrintConv} eq '$self->ConvertDateTime($val)') {
+                    my @g = $exifTool->GetGroup($tagInfo);
+                    warn "$short $name should be in 'Time' group!\n" unless $g[2] eq 'Time';
+                    if ($writable and not $$tagInfo{Shift} and $g[0] ne 'Composite' and
+                        $short ne 'PostScript')
+                    {
+                        warn "$short $name is not shiftable!\n";
+                    }
+                } elsif ($name =~ /DateTime(?!Stamp)/ and (not $$tagInfo{Groups}{2} or
+                    $$tagInfo{Groups}{2} ne 'Time') and $short ne 'DICOM') {
+                    warn "$short $name should be in 'Time' group!\n";
+                }
                 # validate Description (can't contain special characters)
                 if ($$tagInfo{Description} and
                     $$tagInfo{Description} ne EscapeHTML($$tagInfo{Description}))
@@ -575,12 +601,6 @@ TagID:  foreach $tagID (@keys) {
                     warn "Warning: SubIFD flag not set for $short $name\n" if $isSub;
                 }
 
-                my $writable;
-                if (defined $$tagInfo{Writable}) {
-                    $writable = $$tagInfo{Writable};
-                } elsif (not $$tagInfo{SubDirectory}) {
-                    $writable = $$table{WRITABLE};
-                }
                 if ($$tagInfo{Notes}) {
                     my $note = $$tagInfo{Notes};
                     # remove leading/trailing blank lines
@@ -618,8 +638,18 @@ TagID:  foreach $tagID (@keys) {
                 }
                 my $printConv = $$tagInfo{PrintConv};
                 if ($$tagInfo{Mask}) {
-                    push @values, sprintf('[Mask 0x%x]',$$tagInfo{Mask});
+                    my $val = $$tagInfo{Mask};
+                    push @values, sprintf('[Mask 0x%.2x]',$val);
                     $$tagInfo{PrintHex} = 1 unless defined $$tagInfo{PrintHex};
+                    # verify that all values are within the mask
+                    if (ref $printConv eq 'HASH') {
+                        # convert mask if necessary
+                        $val = eval $$tagInfo{ValueConv} if $$tagInfo{ValueConv};
+                        foreach (keys %$printConv) {
+                            next if $_ !~ /^\d+$/ or ($_ & $val) == $_;
+                            warn "$short $name PrintConv value $_ is not in Mask!\n";
+                        }
+                    }
                 }
                 if (ref($printConv) =~ /^(HASH|ARRAY)$/) {
                     my (@printConvList, @indexList, $index);
@@ -1183,6 +1213,7 @@ sub GetTableOrder()
         IPTC    => 'Exif',  # put IPTC after EXIF,
         GPS     => 'XMP',   # etc...
         GeoTiff => 'GPS',
+        CanonVRD=> 'CanonCustom',
         Kodak   => 'JVC',
        'Kodak::IFD' => 'Kodak::Unknown',
        'Kodak::TextualInfo' => 'Kodak::IFD',
@@ -1635,7 +1666,14 @@ sub WriteTagNames($$)
                 $subdir = 1;
                 @vals = @$values;
                 # remove Notes if subdir has Notes as well
-                shift @vals if $vals[0] =~ /^\(/ and @vals >= @$writable;
+                if ($vals[0] =~ /^\(/ and @vals >= @$writable) {
+                    shift @vals;
+                }
+                if ($vals[0] =~ /^\[Mask (0x[\da-f]+)\]/) {
+                    # combine any mask into the format string
+                    $wrStr = "$wrStr & $1" if $wrStr;
+                    shift @vals;
+                }
                 foreach (@vals) { /^\(/ and $_ = '-' }
                 my $i;  # fill in any missing entries from non-directory tags
                 for ($i=0; $i<@$writable; ++$i) {
@@ -1648,6 +1686,11 @@ sub WriteTagNames($$)
                     $wrStr = $vals[0];
                 }
                 shift @vals;
+            } elsif ($wrStr and $wrStr ne 'N' and (
+                (@$values > 0 and $$values[0] =~ /^\[Mask (0x[\da-f]+)\]/) or
+                (@$values > 1 and $$values[1] =~ /^\[Mask (0x[\da-f]+)\]/)))
+            {
+                $wrStr = "$wrStr & $1"; # add Mask to Writable column
             }
             my $tag = shift @tags;
             printf PODFILE "%s%-${wTag}s", $idStr, $tag;
@@ -1727,11 +1770,14 @@ sub WriteTagNames($$)
                 if ($isSubdir) {
                     my ($smallNote, @values);
                     foreach (@$values) {
-                        if (/^[[(]/) {
+                        if (/^\(/) {
+                            # set the note font
                             $smallNote = 1 if $numTags < 2;
                             push @values, ($smallNote ? $noteFontSmall : $noteFont) . "$_</span>";
                             next;
                         }
+                        # make text in square brackets small
+                        /^\[/ and push(@values, "<span class=s>$_</span>"), next;
                         /=/ and push(@values, $_), next;
                         my @names = split;
                         $url = (shift @names) . '.html';
