@@ -18,7 +18,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.02';
+$VERSION = '1.03';
 
 sub ParseAnt($);
 sub ProcessAnt($$$);
@@ -62,7 +62,6 @@ sub ProcessBZZ($$$);
     2 => {
         Name => 'ImageHeight',
         Format => 'int16u',
-        
     },
     4 => {
         Name => 'DjVuVersion',
@@ -175,13 +174,13 @@ sub ProcessBZZ($$$);
         Name => 'CreateDate',
         Groups => { 2 => 'Time' },
         # RFC 3339 date/time format
-        ValueConv => 'use Image::ExifTool::XMP; Image::ExifTool::XMP::ConvertXMPDate($val)',
+        ValueConv => 'require Image::ExifTool::XMP; Image::ExifTool::XMP::ConvertXMPDate($val)',
         PrintConv => '$self->ConvertDateTime($val)',
     },
     ModDate => {
         Name => 'ModifyDate',
         Groups => { 2 => 'Time' },
-        ValueConv => 'use Image::ExifTool::XMP; Image::ExifTool::XMP::ConvertXMPDate($val)',
+        ValueConv => 'require Image::ExifTool::XMP; Image::ExifTool::XMP::ConvertXMPDate($val)',
         PrintConv => '$self->ConvertDateTime($val)',
     },
     Trapped => {
