@@ -15,7 +15,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.03';
+$VERSION = '1.04';
 
 sub ProcessMPImageList($$$);
 
@@ -93,7 +93,9 @@ sub ProcessMPImageList($$$);
     GROUPS => { 0 => 'MPF', 1 => 'MPImage', 2 => 'Image'},
     NOTES => q{
         The first MPF "Large Thumbnail" image is extracted as PreviewImage, and the
-        rest of the embedded MPF images are extracted as MPImage#.
+        rest of the embedded MPF images are extracted as MPImage#.  The
+        ExtractEmbedded option may be used to extract information from these
+        embedded images.
     },
     0.1 => {
         Name => 'MPImageFlags',
