@@ -15,7 +15,7 @@ package Image::ExifTool::Shortcuts;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.29';
+$VERSION = '1.30';
 
 # this is a special table used to define command-line shortcuts
 %Image::ExifTool::Shortcuts::Main = (
@@ -85,9 +85,7 @@ $VERSION = '1.29';
         'ISO',
         'Lens',
         'FocalLength',
-        #'ImageSize', #wrong in CR2
-        'ExifImageWidth', #instead
-        'ExifImageHeight', #instead
+        'ImageSize',
         'Quality',
         'Flash',
         'FlashType',
