@@ -16,6 +16,7 @@
 
 package Image::ExifTool::XMP;
 
+use strict;
 use Image::ExifTool qw(:Utils);
 use Image::ExifTool::XMP;
 
@@ -725,7 +726,21 @@ my %plusVocab = (
     %xmpTableDefaults,
     GROUPS => { 0 => 'XMP', 1 => 'XMP-acdsee', 2 => 'Image' },
     NAMESPACE => 'acdsee',
-    NOTES => 'ACD Systems ACDSee schema tags.',
+    NOTES => q{
+        ACD Systems ACDSee schema tags.  To software developers: Re-inventing your
+        own private tags instead of using the equivalent tags in standard XMP
+        namespaces defeats one of the most valuable features of metadata -- the
+        ability to transfer information.  Your applications mumble to themselves
+        instead of speaking out for the rest of the world to hear.
+    },
+    author     => { Avoid => 1, Groups => { 2 => 'Author' } },
+    caption    => { Avoid => 1 },
+    categories => { Avoid => 1 },
+    datetime   => { Avoid => 1, Groups => { 2 => 'Time' }, %dateTimeInfo },
+    keywords   => { Avoid => 1, List => 'Bag' },
+    notes      => { Avoid => 1 },
+    rating     => { Avoid => 1, Writable => 'real' }, # integer?
+    tagged     => { Avoid => 1, Writable => 'boolean' },
     rpp => {
         Name => 'RPP',
         Writable => 'lang-alt',
@@ -839,7 +854,11 @@ my %plusVocab = (
 
 # set "Avoid" flag for all PRISM tags
 my ($table, $key);
-foreach $table (\%prism, \%prl, \%pur) {
+foreach $table (
+    \%Image::ExifTool::XMP::prism,
+    \%Image::ExifTool::XMP::prl,
+    \%Image::ExifTool::XMP::pur)
+{
     foreach $key (TagTableKeys($table)) {
         $table->{$key}->{Avoid} = 1;
     }
